@@ -6,6 +6,7 @@ import gi
 
 gi.require_version("Gtk", "3.0")
 from gi.repository import Gdk, GdkPixbuf, Gio, Gtk, Pango
+from ks_includes.widgets.scroll import CustomScrolledWindow
 
 
 def find_widget(widget, wanted_type):
@@ -34,12 +35,11 @@ class KlippyGtk:
     def __init__(self, screen):
         self.screen = screen
         self.themedir = os.path.join(pathlib.Path(__file__).parent.resolve().parent, "styles", screen.theme, "images")
-        self.cursor = screen.show_cursor
         self.font_size_type = screen._config.get_main_config().get("font_size", "medium")
         self.width = screen.width
         self.height = screen.height
         self.ultra_tall = (self.height / self.width) >= 3
-        self.font_ratio = [31, 47] if self.screen.vertical_mode else [43, 29]
+        self.font_ratio = [28, 42] if self.screen.vertical_mode else [40, 27]
         self.font_size = min(self.width / self.font_ratio[0], self.height / self.font_ratio[1])
         self.img_scale = self.font_size * 2
         self.button_image_scale = 1.38
@@ -47,15 +47,16 @@ class KlippyGtk:
         self.dialog_buttons_height = round(self.height / 5)
 
         if self.font_size_type == "max":
-            self.font_size = self.font_size * 1.2
+            self.font_size = self.font_size * 1.06
+            self.img_scale = self.img_scale * 0.7
             self.bsidescale = .7
         elif self.font_size_type == "extralarge":
-            self.font_size = self.font_size * 1.14
+            self.font_size = self.font_size * 1.05
             self.img_scale = self.img_scale * 0.7
-            self.bsidescale = 1
+            self.bsidescale = 1.0
         elif self.font_size_type == "large":
-            self.font_size = self.font_size * 1.09
-            self.img_scale = self.img_scale * 0.9
+            self.font_size = self.font_size * 1.025
+            self.img_scale = self.img_scale * 0.85
             self.bsidescale = .8
         elif self.font_size_type == "small":
             self.font_size = self.font_size * 0.91
@@ -118,6 +119,9 @@ class KlippyGtk:
         pixbuf = self.PixbufFromIcon(image_name, width, height)
         return Gtk.Image.new_from_pixbuf(pixbuf) if pixbuf is not None else Gtk.Image()
 
+    def update_themedir(self, theme):
+        self.themedir = os.path.join(pathlib.Path(__file__).parent.resolve().parent, "styles", theme, "images")
+
     def PixbufFromIcon(self, filename, width=None, height=None):
         width = width if width is not None else self.img_width
         height = height if height is not None else self.img_height
@@ -152,12 +156,13 @@ class KlippyGtk:
         return pixbuf
 
     def Button(self, image_name=None, label=None, style=None, scale=None, position=Gtk.PositionType.TOP, lines=2):
-        if self.font_size_type == "max" and label is not None and scale is None:
+        if self.font_size_type == "max" and label is not None:
             image_name = None
         b = Gtk.Button(hexpand=True, vexpand=True, can_focus=False, image_position=position, always_show_image=True)
         if label is not None:
             b.set_label(label.replace("\n", " "))
         if image_name is not None:
+            b.set_name(image_name)
             if scale is None:
                 scale = self.button_image_scale
             if label is None:
@@ -204,11 +209,12 @@ class KlippyGtk:
     def Dialog(self, title, buttons, content, callback=None, *args):
         dialog = Gtk.Dialog(title=title, modal=True, transient_for=self.screen,
                             default_width=self.width, default_height=self.height)
+        dialog.set_size_request(self.width, self.height)
         if not self.screen.windowed:
             dialog.fullscreen()
 
         if buttons:
-            max_buttons = 3 if self.screen.vertical_mode else 4
+            max_buttons = 4
             if len(buttons) > max_buttons:
                 buttons = buttons[:max_buttons]
             if len(buttons) > 2:
@@ -235,20 +241,15 @@ class KlippyGtk:
         dialog.get_style_context().add_class("dialog")
 
         content_area = dialog.get_content_area()
-        content_area.set_margin_start(15)
-        content_area.set_margin_end(15)
-        content_area.set_margin_top(15)
-        content_area.set_margin_bottom(15)
+        content_area.set_margin_start(10)
+        content_area.set_margin_end(5)
+        content_area.set_margin_top(5)
+        content_area.set_margin_bottom(0)
         content_area.add(content)
 
         dialog.show_all()
         # Change cursor to blank
-        if self.cursor:
-            dialog.get_window().set_cursor(
-                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.ARROW))
-        else:
-            dialog.get_window().set_cursor(
-                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.BLANK_CURSOR))
+        self.set_cursor(show=self.screen.show_cursor, window=dialog.get_window())
 
         self.screen.dialogs.append(dialog)
         logging.info(f"Showing dialog {dialog.get_title()} {dialog.get_size()}")
@@ -264,11 +265,16 @@ class KlippyGtk:
             return
         logging.debug(f"Cannot remove dialog {dialog}")
 
-    def ScrolledWindow(self, steppers=True):
-        scroll = Gtk.ScrolledWindow(hexpand=True, vexpand=True, overlay_scrolling=False)
-        scroll.add_events(Gdk.EventMask.BUTTON_PRESS_MASK |
-                          Gdk.EventMask.TOUCH_MASK |
-                          Gdk.EventMask.BUTTON_RELEASE_MASK)
-        if self.screen._config.get_main_config().getboolean("show_scroll_steppers", fallback=False) and steppers:
-            scroll.get_vscrollbar().get_style_context().add_class("with-steppers")
-        return scroll
+    def ScrolledWindow(self, steppers=True, **kwargs):
+        steppers = steppers and self.screen._config.get_main_config().getboolean("show_scroll_steppers", fallback=False)
+        return CustomScrolledWindow(steppers, **kwargs)
+
+    def set_cursor(self, show: bool, window: Gdk.Window):
+        if show:
+            window.set_cursor(
+                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.ARROW))
+            os.system("xsetroot  -cursor_name  arrow")
+        else:
+            window.set_cursor(
+                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.BLANK_CURSOR))
+            os.system("xsetroot  -cursor ks_includes/emptyCursor.xbm ks_includes/emptyCursor.xbm")
