@@ -12,7 +12,7 @@ from ks_includes.screen_panel import ScreenPanel
 
 
 class BasePanel(ScreenPanel):
-    def __init__(self, screen, title):
+    def __init__(self, screen, title=None):
         super().__init__(screen, title)
         self.current_panel = None
         self.time_min = -1
@@ -35,9 +35,7 @@ class BasePanel(ScreenPanel):
         self.control['estop'].connect("clicked", self.emergency_stop)
         self.control['estop'].set_no_show_all(True)
         self.shutdown = {
-            "name": None,
             "panel": "shutdown",
-            "icon": "shutdown",
         }
         self.control['shutdown'] = self._gtk.Button('shutdown', scale=abscale)
         self.control['shutdown'].connect("clicked", self.menu_item_clicked, self.shutdown)
@@ -47,7 +45,6 @@ class BasePanel(ScreenPanel):
         self.control['printer_select'].set_no_show_all(True)
 
         self.shorcut = {
-            "name": "Macros",
             "panel": "gcode_macros",
             "icon": "custom-script",
         }
@@ -111,6 +108,16 @@ class BasePanel(ScreenPanel):
 
         self.update_time()
 
+    def reload_icons(self):
+        button: Gtk.Button
+        for button in self.action_bar.get_children():
+            img = button.get_image()
+            name = button.get_name()
+            pixbuf = img.get_pixbuf()
+            width = pixbuf.get_width()
+            height = pixbuf.get_height()
+            button.set_image(self._gtk.Image(name, width, height))
+
     def show_heaters(self, show=True):
         try:
             for child in self.control['temp_box'].get_children():
@@ -192,6 +199,7 @@ class BasePanel(ScreenPanel):
         self.control['shutdown'].set_visible(not printing)
         self.show_shortcut(connected)
         self.show_heaters(connected)
+        self.show_printer_select(len(self._config.get_printers()) > 1)
         for control in ('back', 'home'):
             self.set_control_sensitive(len(self._screen._cur_panels) > 1, control=control)
         self.current_panel = panel
@@ -301,8 +309,12 @@ class BasePanel(ScreenPanel):
 
     def set_title(self, title):
         self.titlebar.get_style_context().remove_class("message_popup_error")
+        if self._screen.connecting_to_printer != "Printer":
+            printer = self._screen.connecting_to_printer
+        else:
+            printer = ""
         if not title:
-            self.titlelbl.set_label(f"{self._screen.connecting_to_printer}")
+            self.titlelbl.set_label(f"{printer}")
             return
         try:
             env = Environment(extensions=["jinja2.ext.i18n"], autoescape=True)
@@ -312,7 +324,7 @@ class BasePanel(ScreenPanel):
         except Exception as e:
             logging.debug(f"Error parsing jinja for title: {title}\n{e}")
 
-        self.titlelbl.set_label(f"{self._screen.connecting_to_printer} | {title}")
+        self.titlelbl.set_label(f"{printer} {title}")
 
     def update_time(self):
         now = datetime.now()
