@@ -203,12 +203,15 @@ class Panel(MenuPanel):
         return max(temp, 0)
 
     def pid_calibrate(self, temp):
+        heater = self.active_heater.split(' ', maxsplit=1)[-1]
         if self.verify_max_temp(temp):
-            script = {"script": f"PID_CALIBRATE HEATER={self.active_heater} TARGET={temp}"}
+            script = {"script": f"PID_CALIBRATE HEATER={heater} TARGET={temp}"}
             self._screen._confirm_send_action(
                 None,
-                _("Initiate a PID calibration for:") + f" {self.active_heater} @ {temp} ºC"
-                + "\n\n" + _("It may take more than 5 minutes depending on the heater power."),
+                _("Initiate a PID calibration for:")
+                + f" {heater} @ {temp} ºC"
+                + "\n\n"
+                + _("It may take more than 5 minutes depending on the heater power."),
                 "printer.gcode.script",
                 script
             )
@@ -220,10 +223,6 @@ class Panel(MenuPanel):
 
         name = Gtk.Label()
         temp = Gtk.Label(label=_("Temp (°C)"))
-        if self._show_heater_power:
-            temp.get_style_context().add_class("heater-grid-temp-power")
-        else:
-            temp.get_style_context().add_class("heater-grid-temp")
 
         self.labels['devices'].attach(name, 0, 0, 1, 1)
         self.labels['devices'].attach(temp, 1, 0, 1, 1)
