@@ -2,8 +2,10 @@ import logging
 import gi
 
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk, Pango
+from gi.repository import Gtk, Pango, GLib
 from ks_includes.screen_panel import ScreenPanel
+from ks_includes.KlippyGtk import find_widget
+from datetime import datetime
 
 
 class Panel(ScreenPanel):
@@ -12,18 +14,42 @@ class Panel(ScreenPanel):
     distance = distances[-2]
 
     def __init__(self, screen, title):
+        title = title or _("Z Calibrate")
         super().__init__(screen, title)
+        self.last_drop_time = datetime.now()
+        self.initialize_mesh_params()
+        self.initialize_probe_params()
+        self.setup_ui()
+
+    def initialize_mesh_params(self):
         self.mesh_min = []
         self.mesh_max = []
         self.mesh_radius = None
         self.mesh_origin = [0, 0]
         self.zero_ref = []
+        if "BED_MESH_CALIBRATE" not in self._printer.available_commands:
+            return
+        mesh = self._printer.get_config_section("bed_mesh")
+        if 'mesh_radius' in mesh:
+            self.mesh_radius = float(mesh['mesh_radius'])
+            if 'mesh_origin' in mesh:
+                self.mesh_origin = self._csv_to_array(mesh['mesh_origin'])
+        elif 'mesh_min' in mesh and 'mesh_max' in mesh:
+            self.mesh_min = self._csv_to_array(mesh['mesh_min'])
+            self.mesh_max = self._csv_to_array(mesh['mesh_max'])
+        elif 'min_x' in mesh and 'min_y' in mesh and 'max_x' in mesh and 'max_y' in mesh:
+            self.mesh_min = [float(mesh['min_x']), float(mesh['min_y'])]
+            self.mesh_max = [float(mesh['max_x']), float(mesh['max_y'])]
+        if 'zero_reference_position' in self._printer.get_config_section("bed_mesh"):
+            self.zero_ref = self._csv_to_array(mesh['zero_reference_position'])
+
+    def initialize_probe_params(self):
         self.z_hop_speed = 15.0
         self.z_hop = 5.0
         self.probe = self._printer.get_probe()
         if self.probe:
-            self.x_offset = float(self.probe['x_offset']) if "x_offset" in self.probe else 0.0
-            self.y_offset = float(self.probe['y_offset']) if "y_offset" in self.probe else 0.0
+            self.x_offset = float(self.probe.get('x_offset', 0.0))
+            self.y_offset = float(self.probe.get('y_offset', 0.0))
             self.z_offset = float(self.probe['z_offset'])
             if "sample_retract_dist" in self.probe:
                 self.z_hop = float(self.probe['sample_retract_dist'])
@@ -34,20 +60,25 @@ class Panel(ScreenPanel):
             self.y_offset = 0.0
             self.z_offset = 0.0
         logging.info(f"Offset X:{self.x_offset} Y:{self.y_offset} Z:{self.z_offset}")
-        self.widgets['zposition'] = Gtk.Label(label="Z: ?")
 
+    def setup_ui(self):
+        self.widgets['zposition'] = Gtk.Label(label="Z: ?")
         self.widgets['zoffset'] = Gtk.Label(label="?")
+
         pos = Gtk.Grid(row_homogeneous=True, column_homogeneous=True)
         pos.attach(self.widgets['zposition'], 0, 1, 2, 1)
+
         if self.probe:
             pos.attach(Gtk.Label(label=_("Probe Offset") + ": "), 0, 2, 2, 1)
             pos.attach(Gtk.Label(label=_("Saved")), 0, 3, 1, 1)
             pos.attach(Gtk.Label(label=_("New")), 1, 3, 1, 1)
             pos.attach(Gtk.Label(label=f"{self.z_offset:.3f}"), 0, 4, 1, 1)
             pos.attach(self.widgets['zoffset'], 1, 4, 1, 1)
+
         for label in pos.get_children():
             if isinstance(label, Gtk.Label):
                 label.set_ellipsize(Pango.EllipsizeMode.END)
+
         self.buttons = {
             'zpos': self._gtk.Button('z-farther', _("Raise Nozzle"), 'color4'),
             'zneg': self._gtk.Button('z-closer', _("Lower Nozzle"), 'color1'),
@@ -55,17 +86,28 @@ class Panel(ScreenPanel):
             'complete': self._gtk.Button('complete', _('Accept'), 'color3'),
             'cancel': self._gtk.Button('cancel', _('Abort'), 'color2'),
         }
+
         self.buttons['zpos'].connect("clicked", self.move, "+")
         self.buttons['zneg'].connect("clicked", self.move, "-")
         self.buttons['complete'].connect("clicked", self.accept)
         script = {"script": "ABORT"}
-        self.buttons['cancel'].connect("clicked", self._screen._confirm_send_action,
-                                       _("Are you sure you want to stop the calibration?"),
-                                       "printer.gcode.script", script)
+        self.buttons['cancel'].connect(
+            "clicked",
+            self._screen._confirm_send_action,
+            ("Are you sure you want to stop the calibration?"),
+            "printer.gcode.script",
+            script
+        )
+        self.buttons['start'].connect("clicked", self.start_calibration)
 
-        self.popover = Gtk.Popover(position=Gtk.PositionType.BOTTOM)
-
-        self.set_functions()
+        self.dropdown = Gtk.ComboBox.new_with_model(self.set_commands())
+        self.dropdown.connect("changed", self.on_dropdown_change)
+        self.dropdown.connect("notify::popup-shown", self.on_popup_shown)
+        renderer_text = Gtk.CellRendererText()
+        renderer_text.set_property("ellipsize", Pango.EllipsizeMode.END)
+        self.dropdown.pack_start(renderer_text, True)
+        self.dropdown.add_attribute(renderer_text, "text", 0)
+        self.dropdown.set_active(0)
 
         distgrid = Gtk.Grid()
         for j, i in enumerate(self.distances):
@@ -83,112 +125,103 @@ class Panel(ScreenPanel):
         distances.pack_start(self.widgets['move_dist'], True, True, 0)
         distances.pack_start(distgrid, True, True, 0)
 
+        start_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        start_box.pack_start(self.buttons['start'], True, True, 0)
+        start_box.pack_start(self.dropdown, True, True, 0)
+
         grid = Gtk.Grid(column_homogeneous=True)
         if self._screen.vertical_mode:
-            if self._config.get_config()["main"].getboolean("invert_z", False):
-                grid.attach(self.buttons['zpos'], 0, 2, 1, 1)
-                grid.attach(self.buttons['zneg'], 0, 1, 1, 1)
-            else:
-                grid.attach(self.buttons['zpos'], 0, 1, 1, 1)
-                grid.attach(self.buttons['zneg'], 0, 2, 1, 1)
-            grid.attach(self.buttons['start'], 0, 0, 1, 1)
+            zpos_row, zneg_row = (2, 1) if self._config.get_config()["main"].getboolean("invert_z", False) else (1, 2)
+            grid.attach(self.buttons['zpos'], 0, zpos_row, 1, 1)
+            grid.attach(self.buttons['zneg'], 0, zneg_row, 1, 1)
+            grid.attach(start_box, 0, 0, 1, 1)
             grid.attach(pos, 1, 0, 1, 1)
             grid.attach(self.buttons['complete'], 1, 1, 1, 1)
             grid.attach(self.buttons['cancel'], 1, 2, 1, 1)
             grid.attach(distances, 0, 3, 2, 1)
         else:
-            if self._config.get_config()["main"].getboolean("invert_z", False):
-                grid.attach(self.buttons['zpos'], 0, 1, 1, 1)
-                grid.attach(self.buttons['zneg'], 0, 0, 1, 1)
-            else:
-                grid.attach(self.buttons['zpos'], 0, 0, 1, 1)
-                grid.attach(self.buttons['zneg'], 0, 1, 1, 1)
-            grid.attach(self.buttons['start'], 1, 0, 1, 1)
+            zpos_row, zneg_row = (1, 0) if self._config.get_config()["main"].getboolean("invert_z", False) else (0, 1)
+            grid.attach(self.buttons['zpos'], 0, zpos_row, 1, 1)
+            grid.attach(self.buttons['zneg'], 0, zneg_row, 1, 1)
+            grid.attach(start_box, 1, 0, 1, 1)
             grid.attach(pos, 1, 1, 1, 1)
             grid.attach(self.buttons['complete'], 2, 0, 1, 1)
             grid.attach(self.buttons['cancel'], 2, 1, 1, 1)
             grid.attach(distances, 0, 2, 3, 1)
+
         self.content.add(grid)
 
-    def set_functions(self):
-        functions = []
-        pobox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+    def on_dropdown_change(self, dropdown):
+        iterable = dropdown.get_active_iter()
+        if iterable is None:
+            self._screen.show_popup_message("Unknown error with dropdown")
+            return
+        model = dropdown.get_model()
+        logging.debug(f"Selected {model[iterable][0]}")
 
-        if "Z_ENDSTOP_CALIBRATE" in self._printer.available_commands:
-            self._add_button("Endstop", "endstop", pobox)
-            functions.append("endstop")
-        if "PROBE_CALIBRATE" in self._printer.available_commands:
-            self._add_button("Probe", "probe", pobox)
-            functions.append("probe")
-        if "BED_MESH_CALIBRATE" in self._printer.available_commands:
-            mesh = self._printer.get_config_section("bed_mesh")
-            logging.info(f"Mesh: {mesh}")
-            if 'mesh_radius' in mesh:
-                self.mesh_radius = float(mesh['mesh_radius'])
-                if 'mesh_origin' in mesh:
-                    self.mesh_origin = self._csv_to_array(mesh['mesh_origin'])
-                logging.info(f"Mesh Radius: {self.mesh_radius} Origin: {self.mesh_origin}")
-            else:
-                self.mesh_min = self._csv_to_array(mesh['mesh_min'])
-                self.mesh_max = self._csv_to_array(mesh['mesh_max'])
-            if 'zero_reference_position' in self._printer.get_config_section("bed_mesh"):
-                self.zero_ref = self._csv_to_array(mesh['zero_reference_position'])
-            if "probe" not in functions:
-                # This is used to do a manual bed mesh if there is no probe
-                self._add_button("Bed mesh", "mesh", pobox)
-                functions.append("mesh")
-        if "DELTA_CALIBRATE" in self._printer.available_commands:
-            if "probe" in functions:
-                self._add_button("Delta Automatic", "delta", pobox)
-                functions.append("delta")
-            # Since probes may not be accturate enough for deltas, always show the manual method
-            self._add_button("Delta Manual", "delta_manual", pobox)
-            functions.append("delta_manual")
-        if "AXIS_TWIST_COMPENSATION_CALIBRATE" in self._printer.available_commands:
-            self._add_button("Axis Twist Compensation", "axis_twist", pobox)
-            functions.append("axis_twist")
-
-        self.popover.add(pobox)
-        if len(functions) > 1:
-            self.buttons['start'].connect("clicked", self.on_popover_clicked)
+    def on_popup_shown(self, combo_box, param):
+        if combo_box.get_property("popup-shown"):
+            logging.debug("Dropdown popup show")
+            self.last_drop_time = datetime.now()
         else:
-            self.buttons['start'].connect("clicked", self.start_calibration, functions[0])
-        logging.info(f"Available functions for calibration: {functions}")
+            elapsed = (datetime.now() - self.last_drop_time).total_seconds()
+            if elapsed < 0.2:
+                logging.debug(f"Dropdown closed too fast ({elapsed}s)")
+                GLib.timeout_add(50, self.dropdown_keep_open)
+                return
+            logging.debug("Dropdown popup close")
+
+    def dropdown_keep_open(self):
+        self.dropdown.popup()
+        return False
+
+    def set_commands(self):
+        commands = Gtk.ListStore(str)
+
+        if "PROBE_CALIBRATE" in self._printer.available_commands:
+            commands.append({"PROBE_CALIBRATE"})
+        if "Z_ENDSTOP_CALIBRATE" in self._printer.available_commands:
+            commands.append({"Z_ENDSTOP_CALIBRATE"})
+        if "BED_MESH_CALIBRATE" in self._printer.available_commands:
+            commands.append({"BED_MESH_CALIBRATE METHOD=manual"})
+        if "DELTA_CALIBRATE" in self._printer.available_commands:
+            commands.append({"DELTA_CALIBRATE"})
+            commands.append({"DELTA_CALIBRATE METHOD=manual"})
+        if "AXIS_TWIST_COMPENSATION_CALIBRATE" in self._printer.available_commands:
+            commands.append({"AXIS_TWIST_COMPENSATION_CALIBRATE"})
+
+        # Custom commands
+        if self.ks_printer_cfg is not None:
+            if custom_config := self.ks_printer_cfg.get("zcalibrate_custom_commands", None):
+                custom_commands = [str(i.strip()) for i in custom_config.split(',')]
+                for command in custom_commands:
+                    commands.append({f"{command}"})
+
+        logging.info(f"Available commands for calibration: {[row[0] for row in commands]}")
+        return commands
 
     @staticmethod
     def _csv_to_array(string):
         return [float(i.strip()) for i in string.split(',')]
 
-    def _add_button(self, label, method, pobox):
-        popover_button = self._gtk.Button(label=label)
-        popover_button.connect("clicked", self.start_calibration, method)
-        pobox.pack_start(popover_button, True, True, 5)
+    def start_calibration(self, widget):
+        iterable = self.dropdown.get_active_iter()
+        if iterable is None:
+            self._screen.show_popup_message("Unknown error with dropdown")
+            return
+        model = self.dropdown.get_model()
+        command = model[iterable][0]
 
-    def on_popover_clicked(self, widget):
-        self.popover.set_relative_to(widget)
-        self.popover.show_all()
-
-    def start_calibration(self, widget, method):
-        self.popover.popdown()
         self.buttons['start'].set_sensitive(False)
+        self.dropdown.set_sensitive(False)
+
+        self._screen._ws.klippy.gcode_script("SET_GCODE_OFFSET Z=0")
+        if self._printer.config_section_exists("bed_mesh"):
+            self._screen._ws.klippy.gcode_script("BED_MESH_CLEAR")
         if self._printer.get_stat("toolhead", "homed_axes") != "xyz":
             self._screen._ws.klippy.gcode_script("G28")
-        self._screen._ws.klippy.gcode_script("SET_GCODE_OFFSET Z=0")
-        if method == "mesh":
-            self._screen._ws.klippy.gcode_script("BED_MESH_CALIBRATE")
-        else:
-            self._screen._ws.klippy.gcode_script("BED_MESH_CLEAR")
-            if method == "probe":
-                self._move_to_position(*self._get_probe_location())
-                self._screen._ws.klippy.gcode_script("PROBE_CALIBRATE")
-            elif method == "delta":
-                self._screen._ws.klippy.gcode_script("DELTA_CALIBRATE")
-            elif method == "delta_manual":
-                self._screen._ws.klippy.gcode_script("DELTA_CALIBRATE METHOD=manual")
-            elif method == "endstop":
-                self._screen._ws.klippy.gcode_script("Z_ENDSTOP_CALIBRATE")
-            elif method == "axis_twist":
-                self._screen._ws.klippy.gcode_script("AXIS_TWIST_COMPENSATION_CALIBRATE")
+        self._move_to_position(*self._get_calibration_location())
+        self._screen._ws.klippy.gcode_script(command)
 
     def _move_to_position(self, x, y):
         if not x or not y:
@@ -199,7 +232,7 @@ class Panel(ScreenPanel):
         logging.info(f"Moving to X:{x} Y:{y}")
         self._screen._ws.klippy.gcode_script(f'G90\nG0 X{x} Y{y} F3000')
 
-    def _get_probe_location(self):
+    def _get_calibration_location(self):
         if self.ks_printer_cfg is not None:
             x = self.ks_printer_cfg.getfloat("calibrate_x_position", None)
             y = self.ks_printer_cfg.getfloat("calibrate_y_position", None)
@@ -292,6 +325,7 @@ class Panel(ScreenPanel):
     def buttons_calibrating(self):
         self.buttons['start'].get_style_context().remove_class('color3')
         self.buttons['start'].set_sensitive(False)
+        self.dropdown.set_sensitive(False)
 
         self.buttons['zpos'].set_sensitive(True)
         self.buttons['zpos'].get_style_context().add_class('color4')
@@ -305,6 +339,7 @@ class Panel(ScreenPanel):
     def buttons_not_calibrating(self):
         self.buttons['start'].get_style_context().add_class('color3')
         self.buttons['start'].set_sensitive(True)
+        self.dropdown.set_sensitive(True)
 
         self.buttons['zpos'].set_sensitive(False)
         self.buttons['zpos'].get_style_context().remove_class('color4')
