@@ -73,6 +73,7 @@ class KlipperScreen(Gtk.Window):
     windowed = False
     notification_log = []
     prompt = None
+    tempstore_timeout = None
 
     def __init__(self, args):
         self.server_info = None
@@ -145,6 +146,7 @@ class KlipperScreen(Gtk.Window):
         self.change_theme(self.theme)
         self.add(self.base_panel.main_grid)
         self.show_all()
+        self.update_cursor(self.show_cursor)
         min_ver = (3, 8)
         if sys.version_info < min_ver:
             self.show_error_modal(
@@ -157,25 +159,26 @@ class KlipperScreen(Gtk.Window):
         if self._config.errors:
             self.show_error_modal("Invalid config file", self._config.get_errors())
             return
-        if self.show_cursor:
-            self.get_window().set_cursor(
-                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.ARROW))
-            os.system("xsetroot  -cursor_name  arrow")
-        else:
-            self.get_window().set_cursor(
-                Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.BLANK_CURSOR))
-            os.system("xsetroot  -cursor ks_includes/emptyCursor.xbm ks_includes/emptyCursor.xbm")
         self.base_panel.activate()
         self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
         self.log_notification("KlipperScreen Started", 1)
         self.initial_connection()
 
+    def update_cursor(self, show: bool):
+        self.show_cursor = show
+        self.gtk.set_cursor(show, window=self.get_window())
+
     def state_execute(self, state, callback):
         self.close_screensaver()
+        if 'printer_select' in self._cur_panels:
+            logging.debug(f"Connected printer chaged {state}")
+            return False
         if state in ("printing", "paused"):
             self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking_printing'))
         else:
             self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
+            for warning in self.printer.warnings:
+                self.show_popup_message(f"Klipper:\n{warning['message']}", level=2)
         callback()
         return False
 
@@ -232,6 +235,8 @@ class KlipperScreen(Gtk.Window):
             self.printers[ind][name]["moonraker_host"],
             self.printers[ind][name]["moonraker_port"],
             self.printers[ind][name]["moonraker_api_key"],
+            self.printers[ind][name]["moonraker_path"],
+            self.printers[ind][name]["moonraker_ssl"],
         )
         self._ws = KlippyWebsocket(
             {
@@ -243,6 +248,8 @@ class KlipperScreen(Gtk.Window):
             self.printers[ind][name]["moonraker_host"],
             self.printers[ind][name]["moonraker_port"],
             self.printers[ind][name]["moonraker_api_key"],
+            self.printers[ind][name]["moonraker_path"],
+            self.printers[ind][name]["moonraker_ssl"],
         )
         if self.files is None:
             self.files = KlippyFiles(self)
@@ -257,7 +264,7 @@ class KlipperScreen(Gtk.Window):
         requested_updates = {
             "objects": {
                 "bed_mesh": ["profile_name", "mesh_max", "mesh_min", "probed_matrix", "profiles"],
-                "configfile": ["config"],
+                "configfile": ["config", "warnings"],
                 "display_status": ["progress", "message"],
                 "fan": ["speed"],
                 "gcode_move": ["extrude_factor", "gcode_position", "homing_origin", "speed_factor", "speed"],
@@ -330,6 +337,8 @@ class KlipperScreen(Gtk.Window):
                 self.panels[panel_name].__init__(self, title, **kwargs)
                 self.panels_reinit.remove(panel_name)
             self._cur_panels.append(panel_name)
+            if 'extra' in kwargs and hasattr(self.panels[panel], "set_extra"):
+                self.panels[panel].set_extra(**kwargs)
             self.attach_panel(panel_name)
         except Exception as e:
             logging.exception(f"Error attaching panel:\n{e}\n\n{traceback.format_exc()}")
@@ -360,6 +369,9 @@ class KlipperScreen(Gtk.Window):
         self.notification_log.append(log_entry)
         self.process_update("notify_log", log_entry)
 
+    def notification_log_clear(self):
+        self.notification_log.clear()
+
     def show_popup_message(self, message, level=3, from_ws=False):
         if from_ws:
             if (datetime.now() - self.last_popup_time).seconds < 1:
@@ -622,6 +634,8 @@ class KlipperScreen(Gtk.Window):
 
         # Avoid leaving a cursor-handle
         close.grab_focus()
+        self.gtk.set_cursor(False, window=self.get_window())
+
         self.screensaver = box
         self.screensaver.show_all()
         self.power_devices(None, self._config.get_main_config().get("screen_off_devices", ""), on=False)
@@ -641,6 +655,7 @@ class KlipperScreen(Gtk.Window):
         for dialog in self.dialogs:
             logging.info(f"Restoring Dialog {dialog}")
             dialog.show()
+        self.gtk.set_cursor(self.show_cursor, window=self.get_window())
         self.show_all()
         self.power_devices(None, self._config.get_main_config().get("screen_on_devices", ""), on=True)
 
@@ -871,6 +886,11 @@ class KlipperScreen(Gtk.Window):
                 self.prompt.decode(action)
             elif data.startswith("echo: "):
                 self.show_popup_message(data[6:], 1, from_ws=True)
+            elif "!! Extrude below minimum temp" in data:
+                if "temperature" != self._cur_panels[-1]:
+                    self.show_panel("temperature", extra=self.printer.get_stat("toolhead", "extruder"))
+                self.show_popup_message(_("Temperature too low to extrude"))
+                return
             elif data.startswith("!! "):
                 self.show_popup_message(data[3:], 3, from_ws=True)
             elif "unknown" in data.lower() and \
@@ -1097,19 +1117,26 @@ class KlipperScreen(Gtk.Window):
 
     def init_tempstore(self):
         if len(self.printer.get_temp_devices()) == 0:
-            return
+            return False
         tempstore = self.apiclient.send_request("server/temperature_store")
         if tempstore:
             self.printer.init_temp_store(tempstore)
             if hasattr(self.panels[self._cur_panels[-1]], "update_graph_visibility"):
                 self.panels[self._cur_panels[-1]].update_graph_visibility()
+            if self.tempstore_timeout:
+                self.remove_tempstore_timeout()
         else:
             logging.error(f'Tempstore not ready: {tempstore} Retrying in 5 seconds')
-            GLib.timeout_add_seconds(5, self.init_tempstore)
-            return
-        if set(self.printer.tempstore) != set(self.printer.get_temp_devices()):
-            GLib.timeout_add_seconds(5, self.init_tempstore)
-            return
+            if self.tempstore_timeout:
+                return False
+            if self.reinit_count < self.max_retries:
+                self.reinit_count += 1
+                self.tempstore_timeout = GLib.timeout_add_seconds(5, self.retry_init_tempstore)
+            else:
+                logging.error("Max retries reached. Stopping attempts to initialize tempstore.")
+                self.remove_tempstore_timeout()
+            return False
+
         server_config = self.apiclient.send_request("server/config")
         if server_config:
             try:
@@ -1119,6 +1146,15 @@ class KlipperScreen(Gtk.Window):
                 logging.error("Couldn't get the temperature store size")
         return False
 
+    def remove_tempstore_timeout(self):
+        GLib.source_remove(self.tempstore_timeout)
+        self.tempstore_timeout = None
+        self.reinit_count = 0
+
+    def retry_init_tempstore(self):
+        self.remove_tempstore_timeout()
+        return self.init_tempstore()
+
     def show_keyboard(self, entry=None, event=None):
         if self.keyboard is not None:
             return
@@ -1184,7 +1220,7 @@ class KlipperScreen(Gtk.Window):
     def update_size(self, *args):
         width, height = self.get_size()
         if width != self.width or height != self.height:
-            logging.info(f"Size changed: {self.width}x{self.height}")
+            logging.info(f"Size changed: {width}x{height}")
         self.width, self.height = width, height
         new_ratio = self.width / self.height
         new_mode = new_ratio < 1.0
